package com.arosii.jmeter.protocol.ihe.sampler;

import static org.apache.jmeter.protocol.http.sampler.HTTPSamplerBase.DOMAIN;
import static org.apache.jmeter.protocol.http.sampler.HTTPSamplerBase.PATH;
import static org.apache.jmeter.protocol.http.sampler.HTTPSamplerBase.PORT;
import static org.apache.jmeter.protocol.http.sampler.HTTPSamplerBase.PROTOCOL;

import java.io.ByteArrayOutputStream;
import java.io.PrintStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import org.apache.jmeter.protocol.http.sampler.HTTPSampleResult;
import org.apache.jmeter.protocol.http.util.HTTPConstants;
import org.apache.jmeter.samplers.AbstractSampler;
import org.apache.jmeter.samplers.Entry;
import org.apache.jmeter.samplers.SampleResult;
import org.apache.jorphan.util.JOrphanUtils;
import org.openehealth.ipf.commons.ihe.ws.WsTransactionConfiguration;
import org.openehealth.ipf.commons.ihe.xds.core.XdsClientFactory;

import com.arosii.jmeter.protocol.ihe.cxf.ContentCollectorInInterceptor;
import com.arosii.jmeter.protocol.ihe.cxf.ContentCollectorOutInterceptor;
import com.arosii.jmeter.protocol.ihe.cxf.DGWSInterceptor;
import com.arosii.jmeter.protocol.ihe.cxf.HSUIDInterceptor;
import com.arosii.jmeter.protocol.ihe.io.ContentCollectorInputStream;
import com.arosii.jmeter.protocol.ihe.io.ContentCollectorOutputStream;

public abstract class AbstractIHESampler<T> extends AbstractSampler {

	public static final String ADD_DGWS_HEADERS = "IHESampler.add_dgws_header";

	public static final String CITIZEN_CPR_NUMBER = "IHESampler.cpr";

	public static final String ORGANISATION_IDENTIFIER = "IHESampler.organisation_identifier";

	public static final String ORGANISATION_IDENTIFIER_FORMAT = "IHESampler.organisation_identifier_format";

	private static ThreadLocal<Map<String, Object>> clientsCache = new ThreadLocal<Map<String, Object>>();

	private boolean useDGWSHeaders() {
		return getPropertyAsBoolean(ADD_DGWS_HEADERS, false);
	}

	protected String getCitizenCPRNumber() {
		return getPropertyAsString(CITIZEN_CPR_NUMBER);
	}

	public String getProtocol() {
		String protocol = getPropertyAsString(PROTOCOL);
		if (protocol == null || protocol.length() == 0) {
			return HTTPConstants.PROTOCOL_HTTP;
		}
		return protocol;
	}

	public String getDomain() {
		return getPropertyAsString(DOMAIN);
	}

	public int getPort() {
		return getPropertyAsInt(PORT, 0);
	}

    public String getPath() {
        return JOrphanUtils.replaceAllChars(getPropertyAsString(PATH), ' ', "%20");
    }

	public URL getUrl() {
		try {
			return new URL(getProtocol(), getDomain(), getPort(), getPath());
		} catch (MalformedURLException e) {
			throw new IllegalArgumentException(e);
		}
	}

	@Override
	public SampleResult sample(Entry entry) {
		SampleResult res = null;
		try {
			res = sample();
			res.setSampleLabel(getName());
			return res;
		} catch (Exception e) {
			return errorResult(e, new HTTPSampleResult());
		}
	}

	protected abstract SampleResult sample() throws Exception;

	protected HTTPSampleResult errorResult(Throwable e, HTTPSampleResult res) {
		res.setSampleLabel("Exception: " + res.getSampleLabel());
		res.setDataType(SampleResult.TEXT);
		ByteArrayOutputStream text = new ByteArrayOutputStream(200);
		e.printStackTrace(new PrintStream(text));
		res.setResponseData(text.toByteArray());
		res.setResponseCode(e.getClass().getName());
		res.setResponseMessage(e.getMessage());
		res.setSuccessful(false);
		res.setMonitor(false);
		return res;
	}

	@SuppressWarnings("unchecked")
	protected T getClient(Class<T> clazz) {
		URL url = getUrl();
		String key = url + clazz.getName() + useDGWSHeaders();
		Map<String, Object> clients = clientsCache.get();
		if (clients == null) {
			clients = new HashMap<String, Object>();
			clientsCache.set(clients);
		}
		T client = (T) clients.get(key);
		if (client == null) {
			client = createClient(url, clazz);
			clients.put(key, client);
		}
		return client;
	}

	@SuppressWarnings("unchecked")
	protected T createClient(URL url, Class<T> clazz) {
		String wsdl = "wsdl/" + clazz.getSimpleName().substring(0, 5).toLowerCase() + ".wsdl";
		WsTransactionConfiguration configuration = getWsTransactionConfiguration(wsdl, clazz);
		Object clientObject = new XdsClientFactory(configuration, url.toExternalForm(), null, null, null).getClient();
		Client client = ClientProxy.getClient(clientObject);
		client.getInInterceptors().add(new ContentCollectorInInterceptor(getThreadContext()));
		client.getOutInterceptors().add(new ContentCollectorOutInterceptor(getThreadContext()));
		if (useDGWSHeaders()) {
			client.getOutInterceptors().add(new DGWSInterceptor(getThreadContext()));
			client.getOutInterceptors().add(new HSUIDInterceptor(getThreadContext()));
		}
		return (T) clientObject;
	}

	protected abstract WsTransactionConfiguration getWsTransactionConfiguration(String wsdl, Class<T> clazz);

	protected ContentCollectorOutputStream getContentCollectorOutputStream() {
		return (ContentCollectorOutputStream) getThreadContext().getVariables().getObject(ContentCollectorOutputStream.class.getName());
	}

	protected ContentCollectorInputStream getContentCollectorInputStream() {
		return (ContentCollectorInputStream) getThreadContext().getVariables().getObject(ContentCollectorInputStream.class.getName());
	}

}
