package com.arosii.jmeter.protocol.ihe.visualizers;

import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.util.stream.Collectors;

import javax.activation.DataHandler;

import org.apache.jmeter.samplers.SampleResult;
import org.apache.jmeter.visualizers.ResultRenderer;
import org.apache.jmeter.visualizers.SamplerResultTab;
import org.apache.jmeter.visualizers.ViewResultsFullVisualizer;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Code;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.DocumentEntry;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.LocalizedString;
import org.openehealth.ipf.commons.ihe.xds.core.responses.ErrorInfo;
import org.openehealth.ipf.commons.ihe.xds.core.responses.QueryResponse;
import org.openehealth.ipf.commons.ihe.xds.core.responses.RetrievedDocument;
import org.openehealth.ipf.commons.ihe.xds.core.responses.RetrievedDocumentSet;

import com.arosii.jmeter.protocol.ihe.sampler.ITI18SampleResult;
import com.arosii.jmeter.protocol.ihe.sampler.ITI43SampleResult;
import com.arosii.jmeter.resources.ihe.Messages;

public class RenderAsIHE extends SamplerResultTab implements ResultRenderer {

    /** {@inheritDoc} */
    @Override
    public void renderResult(SampleResult sampleResult) {
    	StringBuilder iheText = new StringBuilder();
    	if (sampleResult instanceof ITI18SampleResult) {
			ITI18SampleResult iti18SampleResult = (ITI18SampleResult) sampleResult;
			QueryResponse queryResponse = iti18SampleResult.getQueryResponse();
			if (queryResponse != null) {
				iheText.append(queryResponse.getStatus().toString()).append('\n');
				for (ErrorInfo error : queryResponse.getErrors()) {
					iheText.append('\n');
					iheText.append(error).append('\n');
				}
				for (DocumentEntry documentEntry : queryResponse.getDocumentEntries()) {
					iheText.append('\n');
					iheText.append(documentEntry.getTitle().getValue()).append('\n');
					iheText.append(typeCode(documentEntry)).append('\n');
					iheText.append(documentEntry.getRepositoryUniqueId()).append('\n');
					iheText.append(documentEntry.getUniqueId()).append('\n');
				}
			} else {
				iheText.append("NULL");
			}
		} else if (sampleResult instanceof ITI43SampleResult) {
			ITI43SampleResult iti43SampleResult = (ITI43SampleResult) sampleResult;
			RetrievedDocumentSet retrievedDocumentSet = iti43SampleResult.getRetrievedDocumentSet();
			if (retrievedDocumentSet != null) {
				iheText.append(retrievedDocumentSet.getStatus().toString()).append('\n');
				for (ErrorInfo error : retrievedDocumentSet.getErrors()) {
					iheText.append('\n');
					iheText.append(error).append('\n');
				}
				for (RetrievedDocument retrievedDocument : retrievedDocumentSet.getDocuments()) {
					iheText.append('\n');
					iheText.append(retrievedDocument.getNewRepositoryUniqueId()).append('\n');
					iheText.append(retrievedDocument.getNewDocumentUniqueId()).append('\n');
					DataHandler data = retrievedDocument.getDataHandler();
					iheText.append(data.getContentType()).append('\n');
					length(iheText, data);
				}
			} else {
				iheText.append("NULL");
			}
		}
    	if (iheText.length() > 0) {
    		showTextResponse(iheText.toString());
    	} else {
    		showTextResponse(ViewResultsFullVisualizer.getResponseAsString(sampleResult));
    	}
    }

	private String typeCode(DocumentEntry documentEntry) {
		Code typeCode = documentEntry.getTypeCode();
		StringBuilder sb = new StringBuilder();
		if (typeCode != null) {
			LocalizedString displayName = typeCode.getDisplayName();
			if (displayName != null) {
				sb.append(displayName.getValue()).append(' ');
			}
			sb.append('(').append(typeCode.getCode()).append(" / ").append(typeCode.getSchemeName()).append(')');
		}
		return sb.toString();
	}

	private void length(StringBuilder iheText, DataHandler data) {
		try {
			String documentString = new BufferedReader(new InputStreamReader(data.getInputStream())).lines().collect(Collectors.joining());
			iheText.append(documentString.length()).append('\n');
		} catch (IOException e) {
			ByteArrayOutputStream bytes = new ByteArrayOutputStream();
			try {
				OutputStreamWriter out = new OutputStreamWriter(bytes, "utf-8");
				e.printStackTrace(new PrintWriter(out, true));
			} catch (UnsupportedEncodingException enc) {
				throw new IllegalStateException(enc.getMessage() + " / " + e.getMessage(), e);
			}
			try {
				iheText.append(new String(bytes.toByteArray(), "utf-8"));
			} catch (UnsupportedEncodingException enc) {
				throw new IllegalStateException(enc);
			}
		}
	}

    private void showTextResponse(String response) {
        results.setContentType("text/plain"); // $NON-NLS-1$
        results.setText(response == null ? "" : response); // $NON-NLS-1$
        results.setCaretPosition(0);
        resultsScrollPane.setViewportView(results);
    }

    /** {@inheritDoc} */
    @Override
    public String toString() {
        return Messages.getResString("view_results_render_ihe"); // $NON-NLS-1$
    }

}
